@@ -1,4 +1,4 @@
-dd_SR_loglik = function(pars1,pars2,brts,missnumspec)
+dd_SR_loglik = function(pars1,pars2,brts,missnumspec,methode = 'ode45')
 {
 # brts = branching times (positive, from present to past)
 # - max(brts) = crown age
@@ -31,6 +31,7 @@ dd_SR_loglik = function(pars1,pars2,brts,missnumspec)
 # - pars2[5] = parameters and likelihood should be printed (1) or not (0)
 # - pars2[6] = likelihood is for a tree with crown age (2) or stem age (1)
 # missnumspec = number of missing species    
+# methode = the method used in the numerical solving of the set of the ode's
 
 if(length(pars2) == 4)
 {
@@ -93,7 +94,7 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
               for(k in 2:(kshift-1))
               {
                  k1 = k + (soc - 2)
-                 y = ode(probs,brts[(k-1):k],dd_loglik_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol,method = "lsoda")
+                 y = ode(probs,brts[(k-1):k],dd_loglik_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
                  probs = y[2,2:(lx+1)]
                  if(k < (S + 2 - soc))
                  {
@@ -111,9 +112,9 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
            }   
            k = kshift
            k1 = k + (soc - 2)
-           y = ode(probs,c(brts[k-1],tshift),dd_loglik_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+           y = ode(probs,c(brts[k-1],tshift),dd_loglik_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
            probs = y[2,2:(lx+1)]
-           y = ode(probs,c(tshift,brts[k]),dd_loglik_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+           y = ode(probs,c(tshift,brts[k]),dd_loglik_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
            probs = y[2,2:(lx+1)] 
            if(k < (S + 2 - soc))
            {
@@ -132,7 +133,7 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
               for(k in (kshift + 1):(S + 2 - soc))
               {
                  k1 = k + (soc - 2)
-                 y = ode(probs,brts[(k-1):k],dd_loglik_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+                 y = ode(probs,brts[(k-1):k],dd_loglik_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
                  probs = y[2,2:(lx+1)]
                  if(k < (S + 2 - soc))
                  {
@@ -156,7 +157,7 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
               for(k in (S + 2 - soc):(kshift + 1))
               {
                  k1 = k + (soc - 2)
-                 y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+                 y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
                  probs = y[2,2:(lx+2)]
                  if(k > soc)
                  {
@@ -174,9 +175,9 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
            }
            k = kshift
            k1 = k + (soc - 2)
-           y = ode(probs,-c(brts[k],tshift),dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+           y = ode(probs,-c(brts[k],tshift),dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
            probs = y[2,2:(lx+2)]
-           y = ode(probs,-c(tshift,brts[k-1]),dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+           y = ode(probs,-c(tshift,brts[k-1]),dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
            probs = y[2,2:(lx+2)]
            if(k > soc)
            {
@@ -193,7 +194,7 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
            for(k in (kshift-1):2)
            {
               k1 = k + (soc - 2)
-              y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+              y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
               probs = y[2,2:(lx+2)]
               if(k > soc)
               {
@@ -222,9 +223,9 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
              probs = rep(0,lx)
              probs[1] = 1 # change if other species at crown age
              k = soc
-             y = ode(probs,c(brts[1],tshift),dd_loglik_rhs,c(pars1[1:3],k,ddep),rtol = reltol,atol = abstol, method = "lsoda");
+             y = ode(probs,c(brts[1],tshift),dd_loglik_rhs,c(pars1[1:3],k,ddep),rtol = reltol,atol = abstol, method = methode);
              probs = y[2,2:(lx+1)]
-             y = ode(probs,c(tshift,brts[length(brts)]),dd_loglik_rhs,c(pars1[4:6],k,ddep),rtol = reltol,atol = abstol, method = "lsoda");
+             y = ode(probs,c(tshift,brts[length(brts)]),dd_loglik_rhs,c(pars1[4:6],k,ddep),rtol = reltol,atol = abstol, method = methode);
              probs = y[2,2:(lx+1)]
              if(soc == 1) { aux = 1:lx }
              if(soc == 2) { aux = (2:(lx+1)) * (3:(lx+2))/6 }
@@ -237,16 +238,16 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
              probsn = rep(0,lx + 1)
              probsn[S + missnumspec + 1] = 1
              T = max(1,1/abs(la - mu),1/abs(la2 - mu2)) * 100 * max(abs(brts)) # make this more efficient later
-             y = ode(probsn,c(0,-tshift),dd_loglik_bw_rhs,c(pars1[4:6],0,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+             y = ode(probsn,c(0,-tshift),dd_loglik_bw_rhs,c(pars1[4:6],0,ddep),rtol = reltol,atol = abstol, method = methode)
              probsn = y[2,2:(lx+2)]
-             y = ode(probsn,c(-tshift,T),dd_loglik_bw_rhs,c(pars1[1:3],0,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+             y = ode(probsn,c(-tshift,T),dd_loglik_bw_rhs,c(pars1[1:3],0,ddep),rtol = reltol,atol = abstol, method = methode)
              logliknorm = log(y[2,lx + 2])
              if(soc == 2)
              {
                 probsn = rep(0,lx + 1)
                 probsn[1:lx] = probs[1:lx]
                 probsn = c(flavec(ddep,la,mu,K,0,lx,1,n0),1) * probsn # speciation event
-                y = ode(probsn,c(max(abs(brts)),T),dd_loglik_bw_rhs,c(pars1[1:3],1,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+                y = ode(probsn,c(max(abs(brts)),T),dd_loglik_bw_rhs,c(pars1[1:3],1,ddep),rtol = reltol,atol = abstol, method = methode)
                 logliknorm = logliknorm - log(y[2,lx + 2])
              }
           }
