@@ -1,4 +1,4 @@
-dd_loglik = function(pars1,pars2,brts,missnumspec)
+dd_loglik = function(pars1,pars2,brts,missnumspec,methode = 'ode45')
 {
 # brts = branching times (positive, from present to past)
 # - max(brts) = crown age
@@ -29,6 +29,7 @@ dd_loglik = function(pars1,pars2,brts,missnumspec)
 # - pars2[5] = parameters and likelihood should be printed (1) or not (0)
 # - pars2[6] = likelihood is for a tree with crown age (2) or stem age (1)
 # missnumspec = number of missing species    
+# methode = the method used in the numerical solving of the set of the ode's
 
 if(length(pars2) == 4)
 {
@@ -62,7 +63,10 @@ if((ddep == 1) & ((mu == 0 & missnumspec == 0 & floor(K) != ceiling(K) & la > 0.
 abstol = 1e-16
 reltol = 1e-10 
 brts = -sort(abs(as.numeric(brts)),decreasing = TRUE)
-if(sum(brts == 0) == 0) { brts[length(brts) + 1] = 0 }
+if(sum(brts == 0) == 0)
+{
+   brts[length(brts) + 1] = 0
+}
 
 S = length(brts) + (soc - 2)
 if(min(pars1) < 0)
@@ -86,7 +90,7 @@ if((mu == 0 & (ddep == 2 | ddep == 2.1 | ddep == 2.2)) | (la == 0 & (ddep == 4 |
           for(k in 2:(S + 2 - soc))
           {
              k1 = k + (soc - 2)
-             y = ode(probs,brts[(k-1):k],dd_loglik_rhs,c(pars1,k1,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+             y = ode(probs,brts[(k-1):k],dd_loglik_rhs,c(pars1,k1,ddep),rtol = reltol,atol = abstol,method = methode)
              probs = y[2,2:(lx+1)]
              if(k < (S + 2 - soc))
              {
@@ -107,7 +111,7 @@ if((mu == 0 & (ddep == 2 | ddep == 2.1 | ddep == 2.2)) | (la == 0 & (ddep == 4 |
           for(k in (S + 2 - soc):2)
           {
              k1 = k + (soc - 2)
-             y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1,k1,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+             y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1,k1,ddep),rtol = reltol,atol = abstol,method = methode)
              probs = y[2,2:(lx+2)]
              if(k > soc)
              {
@@ -137,7 +141,7 @@ if((mu == 0 & (ddep == 2 | ddep == 2.1 | ddep == 2.2)) | (la == 0 & (ddep == 4 |
              k = soc
              t1 = brts[1] 
              t2 = brts[S + 2 - soc]
-             y = ode(probsn,c(t1,t2),dd_loglik_rhs,c(pars1,k,ddep),rtol = reltol,atol = abstol, method = "lsoda");
+             y = ode(probsn,c(t1,t2),dd_loglik_rhs,c(pars1,k,ddep),rtol = reltol,atol = abstol,method = methode);
              probsn = y[2,2:(lx+1)]
              if(soc == 1) { aux = 1:lx }
              if(soc == 2) { aux = (2:(lx+1)) * (3:(lx+2))/6 }
@@ -150,14 +154,14 @@ if((mu == 0 & (ddep == 2 | ddep == 2.1 | ddep == 2.2)) | (la == 0 & (ddep == 4 |
              probsn = rep(0,lx + 1)
              probsn[S + missnumspec + 1] = 1
              T = max(1,1/abs(la - mu)) * 100 * max(abs(brts)) # make this more efficient later
-             y = ode(probsn,c(0,T),dd_loglik_bw_rhs,c(pars1,0,ddep),rtol = reltol,atol = abstol, method = "lsoda")
+             y = ode(probsn,c(0,T),dd_loglik_bw_rhs,c(pars1,0,ddep),rtol = reltol,atol = abstol,method = methode)
              logliknorm = log(y[2,lx + 2])
              if(soc == 2)
              {
                 probsn = rep(0,lx + 1)
                 probsn[1:lx] = probs[1:lx]
                 probsn = c(flavec(ddep,la,mu,K,r,lx,1,n0),1) * probsn # speciation event
-                y = ode(probsn,c(max(abs(brts)),T),dd_loglik_bw_rhs,c(pars1,1,ddep),rtol = reltol,atol = abstol,method = "lsoda")
+                y = ode(probsn,c(max(abs(brts)),T),dd_loglik_bw_rhs,c(pars1,1,ddep),rtol = reltol,atol = abstol,method = methode)
                 logliknorm = logliknorm - log(y[2,lx + 2])
              }
           }
